<?php

declare(strict_types=1);

namespace Drupal\access_by_taxonomy;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\field\Entity\FieldConfig;

/**
 * Class AccessByTaxonomyService.
 *
 * Contains generic functionality for the module.
 */
final class AccessByTaxonomyService {

  /**
   * Constructs an AccessByTaxonomyService object.
   */
  public function __construct(
    private readonly EntityFieldManagerInterface $entityFieldManager,
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly LoggerChannelInterface $loggerChannelAccessByTaxonomy,
    private readonly Connection $connection,
  ) {}

  /**
   * Checks whether there are taxonomy fields defined for a given node type.
   */
  public function getNodeTypeTaxonomyTermFieldDefinitions($nodeType): array {
    $field_definitions = $this->entityFieldManager->getFieldDefinitions('node', $nodeType);
    $return = [];
    foreach ($field_definitions as $field_definition) {
      if ($field_definition->getType() === 'entity_reference' && is_numeric(strpos($field_definition->getSetting('handler'), 'taxonomy_term'))) {
        $return[$field_definition->getName()] = $field_definition->getName();
      }
    }
    return $return;
  }

  /**
   * Creates a grants array.
   */
  public function createGrantsArray(string $langcode, string $realm, int $grant_id, int $grant_view, int $grant_update, int $grant_delete, int $nid): array {
    return [
      'langcode' => $langcode,
      'realm' => $realm,
      'gid' => $grant_id,
      'grant_view' => $grant_view,
      'grant_update' => $grant_update,
      'grant_delete' => $grant_delete,
      'nid' => $nid,
    ];
  }

  /**
   * Gets the node access records for a given node.
   */
  public function getNodeAccessRecords(int $id, string $realm = '*'): array {
    $query = $this->connection->select('node_access');
    $query->fields('node_access', ['nid', 'langcode', 'realm', 'gid', 'grant_view']);
    $query->condition('nid', $id);
    $query->condition('realm', $realm);
    try {
      return $query->execute()->fetchAll();
    }
    catch (\Exception $e) {
      $this->loggerChannelAccessByTaxonomy->error($e->getMessage());
      return [];
    }
  }

  /**
   * Adds a field to a vocabulary.
   */
  public function addFieldToVocabulary(string $vocabulary, array $field): void {
    $field_name = array_keys($field)[0];
    $field_label = $field[$field_name];

    // Get the storage for the field storage configuration entity:
    $field_config_storage = $this->entityTypeManager->getStorage('field_storage_config');

    // Load field from configuration.
    $field_storage = $field_config_storage->load('taxonomy_term.' . $field_name);
    if ($field_storage) {
      // Define the field.
      $field = FieldConfig::create([
        'field_storage' => $field_storage,
        'field_name' => $field_name,
        'entity_type' => 'taxonomy_term',
        'bundle' => $vocabulary,
        'label' => $field_label,
      ]);
      // Add it to the entity.
      try {
        $field->save();
      }
      catch (EntityStorageException $e) {
        $this->loggerChannelAccessByTaxonomy->error($e->getMessage());
      }

      // Get the storage for the EntityFormDisplay configuration entity:
      $entity_form_display_storage = $this->entityTypeManager->getStorage('entity_form_display');

      // Set the form display.
      $form_display = $entity_form_display_storage->load('taxonomy_term.' . $vocabulary . '.default');
      if ($form_display) {
        if ($field_name == 'field_allowed_users') {
          $form_display->setComponent($field_name, [
            'type' => 'entity_reference_autocomplete',
          ]);
        }
        else {
          $form_display->setComponent($field_name, [
            'type' => 'options_buttons',
          ]);
        }
        try {
          $form_display->save();
        }
        catch (EntityStorageException $e) {
          $this->loggerChannelAccessByTaxonomy->error($e->getMessage());
        }
      }

      // Set the view display.
      $view_display = $entity_form_display_storage->load('taxonomy_term' . $vocabulary . 'default');
      if ($view_display) {
        $view_display->setComponent($field_name, [
          'label' => 'above',
          'type' => 'entity_reference_label',
        ]);
        try {
          $view_display->save();
        }
        catch (EntityStorageException $e) {
          $this->loggerChannelAccessByTaxonomy->error($e->getMessage());
        }
      }
    }
  }

}
