<?php

namespace Drupal\access_by_taxonomy;

use Drupal\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannel;
use Drupal\node\Entity\Node;
use Drupal\node\NodeGrantDatabaseStorage;

/**
 * Class NodeAccess.
 *
 * This class is responsible for rebuilding node access for terms and nodes.
 */
class NodeAccessService {

  /**
   * Access by Taxonomy service.
   *
   * @var \Drupal\access_by_taxonomy\AccessByTaxonomyService
   */
  protected AccessByTaxonomyService $accessByTaxonomyService;

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Database.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $database;

  /**
   * Node grant storage.
   *
   * @var \Drupal\node\NodeGrantDatabaseStorage
   */
  protected NodeGrantDatabaseStorage $grantStorage;

  /**
   * Logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannel
   */
  protected LoggerChannel $loggerChannel;

  /**
   * RoleGrants service constructor.
   *
   * @param \Drupal\access_by_taxonomy\AccessByTaxonomyService $accessByTaxonomyService
   *   The Access by Taxonomy service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\node\NodeGrantDatabaseStorage $grantStorage
   *   The node grant storage service.
   * @param \Drupal\Core\Logger\LoggerChannel $loggerChannel
   *   The logger factory service.
 */
  final public function __construct(AccessByTaxonomyService $accessByTaxonomyService, EntityTypeManagerInterface $entity_type_manager, Connection $database, NodeGrantDatabaseStorage $grantStorage, LoggerChannel $loggerChannel) {
    $this->accessByTaxonomyService = $accessByTaxonomyService;
    $this->entityTypeManager = $entity_type_manager;
    $this->database = $database;
    $this->grantStorage = $grantStorage;
    $this->loggerChannel = $loggerChannel;
  }

  /**
   * Create service.
   *
   * @param \Drupal\Component\DependencyInjection\ContainerInterface $container
   *   The container.
   *
   * @return static
   *   The service.
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('access_by_taxonomy.access_by_taxonomy_service'),
      $container->get('entity_type.manager'),
      $container->get('database'),
      $container->get('node.grant_storage'),
      $container->get('logger.channel.access_by_taxonomy')
    );
  }

  /**
   * Rebuild access for a node.
   *
   * @param int $nid
   *   Node id value.
   *
   * @return string
   *   Message.
   */
  public static function rebuildNodeAccess(int $nid): string {
    $service = \Drupal::service('access_by_taxonomy.node_access');
    // Delete existing grants for this node.
    $service->database
      ->delete('node_access')
      ->condition('nid', $nid)
      ->execute();
    $service->entityTypeManager->getStorage('node')->resetCache([$nid]);
    $node = Node::load($nid);
    // To preserve database integrity, only write grants if the node
    // loads successfully.
    if (!empty($node)) {
      $grantHandler = $service->entityTypeManager
        ->getAccessControlHandler('node');
      /** @var \Drupal\node\NodeAccessControlHandler $grantHandler */
      $grants = $grantHandler->acquireGrants($node);
      $service->grantStorage->write($node, $grants);
    }

    return 'Processed node ' . $nid;
  }

  /**
   * Rebuild access for nodes using a term.
   *
   * @param int $termId
   *   Taxonomy term id value.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function rebuildTermAccess($termId): void {
    $nids = $this->getNidsUsingTerm($termId);
    $this->loggerChannel->notice('Rebuilding access for term @termId with @count nodes: @nids.', [
      '@termId' => $termId,
      '@count' => count($nids),
      '@nids' => implode(',', $nids),
    ]);
    // Always use batch updating with any node count.
    $operations = array_map(function ($id) {
      return ['Drupal\access_by_taxonomy\NodeAccessService::rebuildNodeAccess', [$id]];
    }, $nids);
    $batch = [
      'title' => t('Updating content access permissions'),
      'operations' => $operations,
      'finished' => 'Drupal\access_by_taxonomy\NodeAccessService::rebuildComplete',
    ];
    batch_set($batch);

  }

  /**
   * Get node ids using a term.
   *
   * @param int $termId
   *   Taxonomy term id value.
   *
   * @return array|int
   *   Array of node ids.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getNidsUsingTerm(int $termId): array|int {
    $node_types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    $fields = [];
    foreach ($node_types as $node_type) {
      $fieldsCt = $this->accessByTaxonomyService->getNodeTypeTaxonomyTermFieldDefinitions((string) $node_type->id());
      $fields = array_merge($fields, $fieldsCt);
    }
    $fields = array_unique($fields);
    $query = $this->entityTypeManager->getStorage('node')->getQuery();
    $or = $query->orConditionGroup();
    foreach ($fields as $field) {
      $or->condition($field, $termId);
    }
    $query->condition($or);
    $query->accessCheck(FALSE);
    return $query->execute();
  }

  /**
   * Rebuild is finished.
   */
  public static function rebuildComplete() {
    // @todo Clear the caches.
  }

}
