<?php

namespace Drupal\access_by_taxonomy;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Class RoleGrants.
 *
 * This class is responsible for managing role grants for the module.
 */
class RoleGrantsService {

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Database.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $database;

  /**
   * RoleGrants service constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, Connection $database) {
    $this->entityTypeManager = $entity_type_manager;
    $this->database = $database;
  }

  /**
   * Populates role grants.
   *
   * If the table is not empty, it does not insert anything.
   * Otherwise, it populates the access_by_taxonomy_role_gid table with roles.
   */
  public function populateRoleGrants():void {
    // Sanity check: if the table is not empty, do not insert anything.
    if (RoleGrantsService::getRoleGrant() !== []) {
      return;
    }
    // Populate the access_by_taxonomy_role_gid table.
    $roles = $this->entityTypeManager->getStorage('user_role')->loadMultiple();
    foreach ($roles as $role) {
      $roleId = $role->id();
      $this->insertRoleGrant((string) $roleId);
    }
  }

  /**
   * Gets role grants.
   *
   * @param array $rids
   *   An array of role IDs.
   *
   * @return array
   *   An array of role grants.
   *
   * @throws \Exception
   */
  public function getRoleGrant(array $rids = []) :array {
    $query = $this->database->select('access_by_taxonomy_role_gid', 't')
      ->fields('t', ['rid', 'gid']);
    if ($rids !== []) {
      $query->condition('t.rid', $rids, 'IN');
    }
    $result = $query->orderBy('t.gid')
      ->execute();
    $grants = [];
    foreach ($result as $grant) {
      $grants[$grant->rid] = $grant->gid;
    }
    return $grants;
  }

  /**
   * Inserts a role grant.
   *
   * @param string $roleId
   *   The role ID.
   *
   * @throws \Exception
   */
  public function insertRoleGrant(string $roleId) : void {
    $fields = [
      'rid' => $roleId,
    ];
    $this->database->insert('access_by_taxonomy_role_gid')
      ->fields($fields)
      ->execute();
  }

  /**
   * Deletes a role grant.
   *
   * @param string $roleId
   *   The role ID.
   */
  public function deleteRoleGrant(string $roleId) : void {
    $this->database->delete('access_by_taxonomy_role_gid')
      ->condition('rid', $roleId)
      ->execute();
  }

}
