<?php

namespace Drupal\Tests\access_by_taxonomy\Kernel;

use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\language\Entity\ConfigurableLanguage;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\Tests\node\Kernel\NodeAccessTestBase;
use Drupal\user\Entity\Role;
use Drupal\user\Entity\User;
use Drupal\user\RoleInterface;

/**
 * Class AccessByTaxonomyKernelTestBase provides common functionality for tests.
 *
 * @package Drupal\Tests\access_by_taxonomy\Kernel
 */
abstract class AccessByTaxonomyKernelTestBase extends NodeAccessTestBase {

  /**
   * Admin user.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $adminUser;

  /**
   * Normal user.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $normalUser;

  /**
   * Normal user.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $editorUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    ConfigurableLanguage::createFromLangcode('en')->save();
    ConfigurableLanguage::createFromLangcode('fr')->save();

    $this->installEntitySchema('user');
    $this->installEntitySchema('node');
    $this->installEntitySchema('taxonomy_term');
    $this->installSchema('access_by_taxonomy', ['access_by_taxonomy_role_gid']);
    $this->installConfig('access_by_taxonomy');
    $this->createTestVocabularies();
    $this->createPageNodeType();
    $this->createUntranslatedNodeType();
    // Give authenticated and anon users permission to access content.
    $this->config('user.role.' . RoleInterface::AUTHENTICATED_ID)
      ->set('permissions', ['access content'])
      ->save();
    $this->config('user.role.' . RoleInterface::ANONYMOUS_ID)
      ->set('permissions', ['access content'])
      ->save();
    $this->adminUser = $this->createAdminUser();
    $this->normalUser = $this->createNormalUser();
    $this->editorUser = $this->createEditorUser();
  }

  /**
   * Creates test vocabularies.
   */
  protected function createTestVocabularies() {
    Vocabulary::create([
      'name' => 'test',
      'vid' => 'test',
    ])->save();

    Vocabulary::create([
      'name' => 'test2',
      'vid' => 'test2',
    ])->save();
  }

  /**
   * Creates a page node type.
   */
  protected function createPageNodeType() {
    FieldStorageConfig::create([
      'entity_type' => 'node',
      'field_name' => 'field_tags',
      'type' => 'entity_reference',
      'settings' => [
        'target_type' => 'taxonomy_term',
      ],
      'cardinality' => FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED,
    ])->save();

    FieldStorageConfig::create([
      'entity_type' => 'node',
      'field_name' => 'field_tags2',
      'type' => 'entity_reference',
      'settings' => [
        'target_type' => 'taxonomy_term',
      ],
      'cardinality' => FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED,
    ])->save();

    FieldConfig::create([
      'field_name' => 'field_tags',
      'entity_type' => 'node',
      'bundle' => 'page',
    ])->save();

    FieldConfig::create([
      'field_name' => 'field_tags2',
      'entity_type' => 'node',
      'bundle' => 'page',
      'translatable' => TRUE,
    ])->save();
  }

  /**
   * Creates a page node type with untranslated fields.
   */
  protected function createUntranslatedNodeType() {
    $this->drupalCreateContentType([
      'type' => 'page_2',
      'name' => 'Page 2',
      'langcode' => LanguageInterface::LANGCODE_NOT_SPECIFIED,
    ]);
    // Add field tags.
    FieldConfig::create([
      'field_name' => 'field_tags',
      'entity_type' => 'node',
      'bundle' => 'page_2',
    ])->save();
  }

  /**
   * Creates a normal user.
   *
   * @return \Drupal\user\Entity\User
   *   The created normal user.
   */
  protected function createNormalUser(): User {
    $testUser = $this->drupalCreateUser(['access content']);

    $testUser->save();

    return $testUser;
  }

  /**
   * Creates a user with editor perms.
   *
   * @return \Drupal\user\Entity\User
   *   The created editor user.
   */
  protected function createEditorUser(): User {
    $role = $this->drupalCreateRole([
      'access content',
      'edit own page content',
      'view own unpublished content',
    ], 'editor');
    $editorUser = $this->drupalCreateUser();
    $editorUser->addRole($role);
    $editorUser->save();

    return $editorUser;
  }

  /**
   * Creates an admin user.
   *
   * @return \Drupal\user\Entity\User|null
   *   The created admin user.
   */
  protected function createAdminUser(): User|NULL {
    if ((Role::load('administrator')) === NULL) {
      $role = [
        'id' => 'administrator',
        'label' => 'administrator',
        'permissions' => [
          'access content',
          'administer content types',
          'administer nodes',
          'bypass node access',
        ],
      ];
      try {
        Role::create($role)->save();
      }
      catch (EntityStorageException $exception) {
        exit($exception->getMessage());
      }
    }
    // User 1 created by NodeAccessTestBase.
    $adminUser = User::load(1);
    return $adminUser;
  }

  /**
   * Creates a term.
   *
   * @param string $name
   *   Name of the term.
   * @param string $langcode
   *   Language code.
   *
   * @return \Drupal\Core\Entity\ContentEntityBase|\Drupal\Core\Entity\EntityBase|\Drupal\Core\Entity\EntityInterface|\Drupal\taxonomy\Entity\Term
   *   The created term.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function createTerm(string $name, $langcode = 'en') {
    $term = Term::create([
      'name'     => $name,
      'vid' => 'test',
      'langcode' => 'en',
    ]);
    $term->save();

    return $term;
  }

}
