<?php

namespace Drupal\Tests\access_by_taxonomy\Kernel;

use Drupal\access_by_taxonomy\NodeAccessService;
use Drupal\Core\Database\Database;
use Drupal\Core\Language\LanguageInterface;
use Drupal\node\Entity\Node;
use Drupal\user\Entity\User;

/**
 * Class PermissionsCheckTest tests access for public and restricted nodes.
 *
 * @package Drupal\Tests\access_by_taxonomy\Kernel
 */
class PermissionsCheckTest extends AccessByTaxonomyKernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'taxonomy',
    'node',
    'user',
    'text',
    'field',
    'system',
    'access_by_taxonomy',
    'language',
    'content_translation',
  ];

  /**
   * Test that everyone can access non-restricted content.
   */
  public function testCanAccessNotRestricted() {
    $term = $this->createTerm('test');

    $node = Node::create([
      'type' => 'page',
      'title' => 'test_title',
      'field_tags' => [
        [
          'target_id' => $term->id(),
        ],
      ],
    ]);
    $node->save();

    $term2 = $this->createTerm(
      'test2',
      LanguageInterface::LANGCODE_NOT_SPECIFIED
    );
    $node2 = Node::create([
      'type' => 'page_2',
      'title' => 'test_title_2',
      'field_tags' => [
        [
          'target_id' => $term2->id(),
        ],
      ],
    ]);
    $node2->save();

    // Check that the node access table is empty for the restricted realm.
    $results = $this->getAccessFromDatabase((int) $node->id(), 'access_by_taxonomy_role', $node->language()->getId());
    $this->assertCount(0, $results);
    $results = $this->getAccessFromDatabase((int) $node2->id(), 'access_by_taxonomy_role', $node2->language()->getId());
    $this->assertCount(0, $results);
    // Check that we have a record for the allowed realm.
    $results = $this->getAccessFromDatabase((int) $node->id(), 'all', $node->language()->getId());
    $this->assertCount(1, $results);
    $results = $this->getAccessFromDatabase((int) $node2->id(), 'all', $node2->language()->getId());
    $this->assertCount(1, $results);
    // Check that we have no record for the user realm.
    $results = $this->getAccessFromDatabase((int) $node->id(), 'access_by_taxonomy_user', $node->language()->getId());
    $this->assertCount(0, $results);
    $results = $this->getAccessFromDatabase((int) $node2->id(), 'access_by_taxonomy_user', $node2->language()->getId());
    $this->assertCount(0, $results);

    $expected_node_access = ['view' => TRUE, 'update' => FALSE, 'delete' => FALSE];
    $this->assertNodeAccess($expected_node_access, $node, User::getAnonymousUser());
    $this->assertNodeAccess($expected_node_access, $node, $this->normalUser);
    $this->assertNodeAccess($expected_node_access, $node, $this->editorUser);
    $this->assertNodeAccess($expected_node_access, $node2, User::getAnonymousUser());
    $this->assertNodeAccess($expected_node_access, $node2, $this->normalUser);
    $this->assertNodeAccess($expected_node_access, $node2, $this->editorUser);
    $expected_all_access = ['view' => TRUE, 'update' => TRUE, 'delete' => TRUE];
    $this->assertNodeAccess($expected_all_access, $node, $this->adminUser);
    $this->assertNodeAccess($expected_all_access, $node2, $this->adminUser);
  }

  /**
   * Test that only admins can access restricted content.
   */
  public function testCanAccessRestricted() {
    $term = $this->createTerm('editor', 'en');
    /** @var \Drupal\taxonomy\Entity\Term $term */
    $term->set('field_allowed_roles', ['target_id' => 'editor']);
    $term->save();
    // En node, en term.
    $node = Node::create([
      'type' => 'page',
      'title' => 'test_title_restricted',
      'field_tags' => [
        [
          'target_id' => $term->id(),
        ],
      ],
    ]);
    $node->save();
    // Und node, und term.
    $term2 = $this->createTerm(
      'editor',
      LanguageInterface::LANGCODE_NOT_SPECIFIED
    );
    /** @var \Drupal\taxonomy\Entity\Term $term2 */
    $term2->set('field_allowed_roles', ['target_id' => 'editor']);
    $term2->save();
    $node2 = Node::create([
      'type' => 'page_2',
      'title' => 'test_title_2',
      'field_tags' => [
        [
          'target_id' => $term2->id(),
        ],
      ],
    ]);
    $node2->save();
    // Und node, en term.
    $node3 = Node::create([
      'type' => 'page_2',
      'title' => 'test_title_2',
      'field_tags' => [
        [
          'target_id' => $term->id(),
        ],
      ],
    ]);
    $node3->save();
    // En node, und term.
    $node4 = Node::create([
      'type' => 'page',
      'title' => 'test_title_4',
      'field_tags' => [
        [
          'target_id' => $term2->id(),
        ],
      ],
    ]);
    $node4->save();

    // Check we have one record for the restricted realm.
    $results = $this->getAccessFromDatabase((int) $node->id(), 'access_by_taxonomy_role', $node->language()->getId());
    $this->assertCount(1, $results);
    $results = $this->getAccessFromDatabase((int) $node2->id(), 'access_by_taxonomy_role', $node2->language()->getId());
    $this->assertCount(1, $results);
    $results = $this->getAccessFromDatabase((int) $node3->id(), 'access_by_taxonomy_role', $node3->language()->getId());
    $this->assertCount(1, $results);
    $results = $this->getAccessFromDatabase((int) $node4->id(), 'access_by_taxonomy_role', $node4->language()->getId());
    $this->assertCount(1, $results);
    // Check there are no entries for the allowed realm.
    $results = $this->getAccessFromDatabase((int) $node->id(), 'all', $node->language()->getId());
    $this->assertCount(0, $results);
    $results = $this->getAccessFromDatabase((int) $node2->id(), 'all', $node2->language()->getId());
    $this->assertCount(0, $results);
    $results = $this->getAccessFromDatabase((int) $node3->id(), 'all', $node3->language()->getId());
    $this->assertCount(0, $results);
    $results = $this->getAccessFromDatabase((int) $node4->id(), 'all', $node4->language()->getId());
    $this->assertCount(0, $results);
    // Check there are no entries for the user realm.
    $results = $this->getAccessFromDatabase((int) $node->id(), 'access_by_taxonomy_user', $node->language()->getId());
    $this->assertCount(0, $results);
    $results = $this->getAccessFromDatabase((int) $node2->id(), 'access_by_taxonomy_user', $node2->language()->getId());
    $this->assertCount(0, $results);
    $results = $this->getAccessFromDatabase((int) $node3->id(), 'access_by_taxonomy_user', $node3->language()->getId());
    $this->assertCount(0, $results);
    $results = $this->getAccessFromDatabase((int) $node4->id(), 'access_by_taxonomy_user', $node4->language()->getId());
    $this->assertCount(0, $results);

    $expected_node_access = ['view' => FALSE, 'update' => FALSE, 'delete' => FALSE];
    $this->assertNodeAccess($expected_node_access, $node, $this->normalUser);
    $this->assertNodeAccess($expected_node_access, $node, User::getAnonymousUser());
    $this->assertNodeAccess($expected_node_access, $node2, $this->normalUser);
    $this->assertNodeAccess($expected_node_access, $node2, User::getAnonymousUser());
    $this->assertNodeAccess($expected_node_access, $node3, $this->normalUser);
    $this->assertNodeAccess($expected_node_access, $node3, User::getAnonymousUser());
    $this->assertNodeAccess($expected_node_access, $node4, $this->normalUser);
    $this->assertNodeAccess($expected_node_access, $node4, User::getAnonymousUser());
    $expected_node_access = ['view' => TRUE, 'update' => FALSE, 'delete' => FALSE];
    $this->assertNodeAccess($expected_node_access, $node, $this->editorUser);
    $this->assertNodeAccess($expected_node_access, $node2, $this->editorUser);
    $this->assertNodeAccess($expected_node_access, $node3, $this->editorUser);
    $this->assertNodeAccess($expected_node_access, $node4, $this->editorUser);
    $expected_all_access = ['view' => TRUE, 'update' => TRUE, 'delete' => TRUE];
    $this->assertNodeAccess($expected_all_access, $node, $this->adminUser);
    $this->assertNodeAccess($expected_all_access, $node2, $this->adminUser);
    $this->assertNodeAccess($expected_all_access, $node3, $this->adminUser);
    $this->assertNodeAccess($expected_all_access, $node4, $this->adminUser);
  }

  /**
   * Test that editors can access restricted own content.
   */
  public function testEditorCanAccessRestrictedOwn() {
    $term = $this->createTerm('administrator');
    /** @var \Drupal\taxonomy\Entity\Term $term */
    $term->set('field_allowed_roles', ['target_id' => 'administrator']);
    $term->save();
    $node = Node::create([
      'type' => 'page',
      'title' => 'test_title_restricted',
      'field_tags' => [
        [
          'target_id' => $term->id(),
        ],
      ],
    ]);

    $editor2 = $this->drupalCreateUser();
    $editor2->addRole('editor');
    $editor2->save();
    $node->setOwner($editor2);
    $node->setUnpublished();
    $node->save();

    // Check that there are 1 entry for the restricted realm.
    $results = $this->getAccessFromDatabase((int) $node->id(), 'access_by_taxonomy_role', $node->language()->getId());
    $this->assertCount(1, $results);
    // Check that there are no entries for the allowed realm.
    $results = $this->getAccessFromDatabase((int) $node->id(), 'all', $node->language()->getId());
    $this->assertCount(0, $results);
    // Check that there are no entries for the user realm.
    $results = $this->getAccessFromDatabase((int) $node->id(), 'access_by_taxonomy_user', $node->language()->getId());
    $this->assertCount(0, $results);

    $expected_node_access = ['view' => FALSE, 'update' => FALSE, 'delete' => FALSE];
    $this->assertNodeAccess($expected_node_access, $node, $this->normalUser);
    $this->assertNodeAccess($expected_node_access, $node, User::getAnonymousUser());
    $this->assertNodeAccess($expected_node_access, $node, $this->editorUser);
    $expected_node_access = ['view' => TRUE, 'update' => TRUE, 'delete' => FALSE];
    $this->assertNodeAccess($expected_node_access, $node, $editor2);
    $expected_all_access = ['view' => TRUE, 'update' => TRUE, 'delete' => TRUE];
    $this->assertNodeAccess($expected_all_access, $node, $this->adminUser);
  }

  /**
   * Test that editors can access unpublished unprotected content.
   */
  public function testEditorCanAccessUnrestrictedOwn() {
    $node = Node::create([
      'type' => 'page',
      'title' => 'test_title_not_restricted',
      'status' => 0,
    ]);
    $node->setOwner($this->editorUser);
    $node->save();

    // Check that there are no entries for the restricted realm.
    $results = $this->getAccessFromDatabase((int) $node->id(), 'access_by_taxonomy_role', $node->language()->getId());
    $this->assertCount(0, $results);
    // Check that there is 0 entry for the allowed realm.
    $results = $this->getAccessFromDatabase((int) $node->id(), 'all', $node->language()->getId());
    $this->assertCount(0, $results);
    // Check that there are no entries for the user realm.
    $results = $this->getAccessFromDatabase((int) $node->id(), 'access_by_taxonomy_user', $node->language()->getId());
    $this->assertCount(0, $results);

    $expected_node_access = ['view' => FALSE, 'update' => FALSE, 'delete' => FALSE];
    $this->assertNodeAccess($expected_node_access, $node, $this->normalUser);
    $this->assertNodeAccess($expected_node_access, $node, User::getAnonymousUser());
    $expected_node_access = ['view' => TRUE, 'update' => TRUE, 'delete' => FALSE];
    $this->assertNodeAccess($expected_node_access, $node, $this->editorUser);
    $expected_all_access = ['view' => TRUE, 'update' => TRUE, 'delete' => TRUE];
    $this->assertNodeAccess($expected_all_access, $node, $this->adminUser);
  }

  /**
   * Tests that when the access changes for a term, all nodes are updated.
   */
  public function testTermAccessChange() {
    /** @var \Drupal\taxonomy\Entity\Term $term */
    $term = $this->createTerm('editor');
    $term->save();
    $node = Node::create([
      'type' => 'page',
      'title' => 'test_title_restricted',
      'field_tags' => [
        [
          'target_id' => $term->id(),
        ],
      ],
    ]);
    $node->save();
    $node2 = Node::create([
      'type' => 'page_2',
      'title' => 'test_title_2',
      'field_tags' => [
        [
          'target_id' => $term->id(),
        ],
      ],
    ]);
    $node2->save();
    $access = $this->getAccessFromDatabase((int) $node->id(), 'all', $node->language()->getId());
    $this->assertCount(1, $access);
    $access2 = $this->getAccessFromDatabase((int) $node2->id(), 'all', $node2->language()->getId());
    $this->assertCount(1, $access2);

    $term->set('field_allowed_roles', ['target_id' => 'editor']);
    $term->save();

    NodeAccessService::rebuildNodeAccess((int) $node->id());
    NodeAccessService::rebuildNodeAccess((int) $node2->id());
    $access3 = $this->getAccessFromDatabase((int) $node->id(), 'access_by_taxonomy_role', $node->language()->getId());
    $access4 = $this->getAccessFromDatabase((int) $node2->id(), 'access_by_taxonomy_role', $node2->language()->getId());

    $this->assertCount(1, $access3);
    $this->assertCount(1, $access4);
  }

  /**
   * Helper function to get out of the database the access for a node.
   *
   * @param int $nid
   *   The node id.
   * @param string $realm
   *   The realm.
   * @param string $langcode
   *   The langcode.
   *
   * @return array
   *   The results from the database.
   */
  private function getAccessFromDatabase(int $nid, string $realm, string $langcode) {
    $results = Database::getConnection()->query('SELECT * FROM {node_access} WHERE nid = :nid AND realm = :realm AND langcode = :langcode',
      [
        ':nid' => $nid,
        ':realm' => $realm,
        ':langcode' => $langcode,
      ])->fetchAll();
    return $results;
  }

}
